/*
 * \brief  Test for superfluous signal-wakeup
 * \author Norman Feske
 * \date   2010-07-05
 */

/* Genode includes */
#include <base/printf.h>
#include <base/signal.h>
#include <base/thread.h>
#include <base/sleep.h>
#include <base/env.h>

using namespace Genode;


/**
 * Set this value to 'true' to trigger the superfluous wakeup
 */
enum { RACE_CONDITION = false };


enum { STACK_SIZE = 4096 };
class Receiver : Thread<STACK_SIZE>
{
	private:

		Signal_receiver &_receiver;

	public:

		Receiver(Signal_receiver &receiver)
		: _receiver(receiver)
		{
			if (RACE_CONDITION)
				start();
		}

		void entry()
		{
			while (1)
			{
				Signal signal = _receiver.wait_for_signal();
				printf("Got signal, num=%d\n", signal.num());
			}
		}

		using Thread_base::start;
};


int main(int, char **)
{
	printf("--- single signal test ---\n");
	Signal_receiver s_rcvr;
	Signal_context ct;
	Receiver *ser_rx = new (env()->heap()) Receiver(s_rcvr);
	Signal_context_capability send_cap = s_rcvr.manage(&ct);

	if (!RACE_CONDITION)
		ser_rx->start();

	Signal_transmitter tx(send_cap);
	tx.submit();

	printf("--- single signal test completed ---\n");
	sleep_forever();
}
